@@ -1,3 +1,10 @@
+//! Combinators that allow combining and extending existing parsers.
+//!
+//! *“Ford... you're turning into a penguin. Stop it.”*
+//!
+//! Although it's *sometimes* useful to be able to name their type, most of these parsers are much easier to work with
+//! when accessed through their respective methods on [`Parser`].
+
 use super::*;
 
 /// See [`Parser::ignored`].
@@ -231,12 +238,100 @@ impl<I: Clone, O, U, A: Parser<I, O, Error = E>, B: Parser<I, U, Error = E>, E:
     }
 }
 
+/// See [`Parser::then_with`]
+pub struct ThenWith<I, O1, O2, A, B, F>(
+    pub(crate) A,
+    pub(crate) F,
+    pub(crate) PhantomData<(I, O1, O2, B)>,
+);
+
+impl<I, O1, O2, A: Clone, B, F: Clone> Clone for ThenWith<I, O1, O2, A, B, F> {
+    fn clone(&self) -> Self {
+        ThenWith(self.0.clone(), self.1.clone(), PhantomData)
+    }
+}
+
+impl<I, O1, O2, A: Copy, B, F: Copy> Copy for ThenWith<I, O1, O2, A, B, F> {}
+
+impl<
+        I: Clone,
+        O1,
+        O2,
+        A: Parser<I, O1, Error = E>,
+        B: Parser<I, O2, Error = E>,
+        F: Fn(O1) -> B,
+        E: Error<I>,
+    > Parser<I, O2> for ThenWith<I, O1, O2, A, B, F>
+{
+    type Error = E;
+
+    #[inline]
+    fn parse_inner<D: Debugger>(
+        &self,
+        debugger: &mut D,
+        stream: &mut StreamOf<I, E>,
+    ) -> PResult<I, O2, E> {
+        let state = stream.save();
+
+        match {
+            #[allow(deprecated)]
+            debugger.invoke(&self.0, stream)
+        } {
+            (mut first_errs, Ok((first_out, first_alt))) => {
+                let second_out = self.1(first_out);
+                match {
+                    #[allow(deprecated)]
+                    debugger.invoke(&second_out, stream)
+                } {
+                    (second_errs, Ok((second_out, second_alt))) => {
+                        first_errs.extend(second_errs);
+                        (first_errs, Ok((second_out, first_alt.or(second_alt))))
+                    }
+                    (second_errs, Err(e)) => {
+                        stream.revert(state);
+                        first_errs.extend(second_errs);
+                        (first_errs, Err(e))
+                    }
+                }
+            }
+            (errs, Err(e)) => {
+                stream.revert(state);
+                (errs, Err(e))
+            }
+        }
+    }
+
+    #[inline]
+    fn parse_inner_verbose(&self, d: &mut Verbose, s: &mut StreamOf<I, E>) -> PResult<I, O2, E> {
+        #[allow(deprecated)]
+        self.parse_inner(d, s)
+    }
+    #[inline]
+    fn parse_inner_silent(&self, d: &mut Silent, s: &mut StreamOf<I, E>) -> PResult<I, O2, E> {
+        #[allow(deprecated)]
+        self.parse_inner(d, s)
+    }
+}
+
 /// See [`Parser::delimited_by`].
 #[derive(Copy, Clone)]
-pub struct DelimitedBy<A, I>(pub(crate) A, pub(crate) I, pub(crate) I);
+pub struct DelimitedBy<A, L, R, U, V> {
+    pub(crate) item: A,
+    pub(crate) start: L,
+    pub(crate) end: R,
+    pub(crate) phantom: PhantomData<(U, V)>,
+}
 
-impl<I: Clone + PartialEq, O, A: Parser<I, O, Error = E>, E: Error<I>> Parser<I, O>
-    for DelimitedBy<A, I>
+impl<
+        I: Clone,
+        O,
+        A: Parser<I, O, Error = E>,
+        L: Parser<I, U, Error = E> + Clone,
+        R: Parser<I, V, Error = E> + Clone,
+        U,
+        V,
+        E: Error<I>,
+    > Parser<I, O> for DelimitedBy<A, L, R, U, V>
 {
     type Error = E;
 
@@ -249,9 +344,11 @@ impl<I: Clone + PartialEq, O, A: Parser<I, O, Error = E>, E: Error<I>> Parser<I,
         // TODO: Don't clone!
         #[allow(deprecated)]
         let (errors, res) = debugger.invoke(
-            &just(self.1.clone())
-                .ignore_then(&self.0)
-                .then_ignore(just(self.2.clone())),
+            &self
+                .start
+                .clone()
+                .ignore_then(&self.item)
+                .then_ignore(self.end.clone()),
             stream,
         );
         (errors, res)
@@ -461,7 +558,7 @@ impl<A, B, U> SeparatedBy<A, B, U> {
     ///     .padded()
     ///     .separated_by(just(','))
     ///     .allow_trailing()
-    ///     .delimited_by('(', ')');
+    ///     .delimited_by(just('('), just(')'));
     ///
     /// assert_eq!(numbers.parse("(1, 2)"), Ok(vec!["1".to_string(), "2".to_string()]));
     /// assert_eq!(numbers.parse("(1, 2,)"), Ok(vec!["1".to_string(), "2".to_string()]));
@@ -828,16 +925,23 @@ impl<I: Clone, O, A: Parser<I, O, Error = E>, U, F: Fn(O, E::Span) -> U, E: Erro
 }
 
 /// See [`Parser::validate`].
-#[derive(Copy, Clone)]
-pub struct Validate<A, F>(pub(crate) A, pub(crate) F);
+pub struct Validate<A, U, F>(pub(crate) A, pub(crate) F, pub(crate) PhantomData<U>);
+
+impl<A: Copy, U, F: Copy> Copy for Validate<A, U, F> {}
+impl<A: Clone, U, F: Clone> Clone for Validate<A, U, F> {
+    fn clone(&self) -> Self {
+        Self(self.0.clone(), self.1.clone(), PhantomData)
+    }
+}
 
 impl<
         I: Clone,
         O,
+        U,
         A: Parser<I, O, Error = E>,
-        F: Fn(O, E::Span, &mut dyn FnMut(E)) -> O,
+        F: Fn(O, E::Span, &mut dyn FnMut(E)) -> U,
         E: Error<I>,
-    > Parser<I, O> for Validate<A, F>
+    > Parser<I, U> for Validate<A, O, F>
 {
     type Error = E;
 
@@ -846,7 +950,7 @@ impl<
         &self,
         debugger: &mut D,
         stream: &mut StreamOf<I, E>,
-    ) -> PResult<I, O, E> {
+    ) -> PResult<I, U, E> {
         let start = stream.save();
         #[allow(deprecated)]
         let (mut errors, res) = debugger.invoke(&self.0, stream);
@@ -865,12 +969,12 @@ impl<
     }
 
     #[inline]
-    fn parse_inner_verbose(&self, d: &mut Verbose, s: &mut StreamOf<I, E>) -> PResult<I, O, E> {
+    fn parse_inner_verbose(&self, d: &mut Verbose, s: &mut StreamOf<I, E>) -> PResult<I, U, E> {
         #[allow(deprecated)]
         self.parse_inner(d, s)
     }
     #[inline]
-    fn parse_inner_silent(&self, d: &mut Silent, s: &mut StreamOf<I, E>) -> PResult<I, O, E> {
+    fn parse_inner_silent(&self, d: &mut Silent, s: &mut StreamOf<I, E>) -> PResult<I, U, E> {
         #[allow(deprecated)]
         self.parse_inner(d, s)
     }
@@ -1284,9 +1388,22 @@ where
 #[cfg(test)]
 mod tests {
     use error::Simple;
+    use text::TextParser;
 
     use super::*;
 
+    #[test]
+    fn delimited_by_complex() {
+        let parser = just::<_, _, Simple<char>>('-')
+            .delimited_by(text::ident().padded(), text::int(10).padded())
+            .separated_by(just(','));
+
+        assert_eq!(
+            parser.parse("one - 1,two - 2,three - 3"),
+            Ok(vec!['-', '-', '-'])
+        );
+    }
+
     #[test]
     fn separated_by_at_least() {
         let parser = just::<_, _, Simple<char>>('-')
