@@ -3,23 +3,14 @@
 #![deny(missing_docs)]
 #![allow(deprecated)] // TODO: Don't allow this
 
-/// Traits that allow chaining parser outputs together.
 pub mod chain;
-/// Combinators that allow combining and extending existing parsers.
 pub mod combinator;
-/// Utilities for debugging parsers.
 pub mod debug;
-/// Error types, traits and utilities.
 pub mod error;
-/// Parser primitives that accept specific token patterns.
 pub mod primitive;
-/// Types and traits that facilitate error recovery.
 pub mod recovery;
-/// Recursive parsers (parser that include themselves within their patterns).
 pub mod recursive;
-/// Types and traits related to spans.
 pub mod span;
-/// Token streams and behaviours.
 pub mod stream;
 pub mod text;
 
@@ -56,6 +47,9 @@ use std::{
 };
 
 /// Commonly used functions, traits and types.
+///
+/// *Listen, three eyes,” he said, “don’t you try to outweird me, I get stranger things than you free with my breakfast
+/// cereal.”*
 pub mod prelude {
     pub use super::{
         error::{Error as _, Simple},
@@ -190,6 +184,7 @@ pub trait Parser<I: Clone, O> {
 
     /// Parse a stream of tokens, yielding an output if possible, and any errors encountered along the way.
     ///
+    /// If `None` is returned (i.e: parsing failed) then there will *always* be at least one item in the error `Vec`.
     /// If you don't care about producing an output if errors are encountered, use [`Parser::parse`] instead.
     ///
     /// Although the signature of this function looks complicated, it's simpler than you think! You can pass a
@@ -206,6 +201,7 @@ pub trait Parser<I: Clone, O> {
     /// Parse a stream of tokens, yielding an output if possible, and any errors encountered along the way. Unlike
     /// [`Parser::parse_recovery`], this function will produce verbose debugging output as it executes.
     ///
+    /// If `None` is returned (i.e: parsing failed) then there will *always* be at least one item in the error `Vec`.
     /// If you don't care about producing an output if errors are encountered, use `Parser::parse` instead.
     ///
     /// Although the signature of this function looks complicated, it's simpler than you think! You can pass a
@@ -407,6 +403,8 @@ pub trait Parser<I: Clone, O> {
 
     /// Validate an output, producing non-terminal errors if it does not fulfil certain criteria.
     ///
+    /// This function also permits mapping the output to a value of another type, similar to [`Parser::map`].
+    ///
     /// If you wish parsing of this pattern to halt when an error is generated instead of continuing, consider using
     /// [`Parser::try_map`] instead.
     ///
@@ -427,12 +425,12 @@ pub trait Parser<I: Clone, O> {
     /// assert_eq!(large_int.parse("537"), Ok(537));
     /// assert!(large_int.parse("243").is_err());
     /// ```
-    fn validate<F>(self, f: F) -> Validate<Self, F>
+    fn validate<F, U>(self, f: F) -> Validate<Self, O, F>
     where
         Self: Sized,
-        F: Fn(O, <Self::Error as Error<I>>::Span, &mut dyn FnMut(Self::Error)) -> O,
+        F: Fn(O, <Self::Error as Error<I>>::Span, &mut dyn FnMut(Self::Error)) -> U,
     {
-        Validate(self, f)
+        Validate(self, f, PhantomData)
     }
 
     /// Label the pattern parsed by this parser for more useful error messages.
@@ -645,6 +643,34 @@ pub trait Parser<I: Clone, O> {
         Then(self, other)
     }
 
+    /// Parse one thing and then another thing, creating the second parser from the result of
+    /// the first. If you only have a couple cases to handle, prefer [`Parser::or`].
+    ///
+    /// The output of this parser is `U`, the result of the second parser
+    ///
+    /// Error recovery for this parser may be sub-optimal, as if the first parser succeeds on
+    /// recovery then the second produces an error, the primary error will point to the location in
+    /// the second parser which failed, ignoring that the first parser may be the root cause. There
+    /// may be other pathological errors cases as well.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// # use chumsky::{prelude::*, error::Cheap};
+    /// let letter_up = one_of::<_, _, Cheap<u8>>((b'a'..=b'z').collect::<Vec<u8>>())
+    ///     .then_with(|letter: u8| just(letter + 1));
+    ///
+    /// assert_eq!(letter_up.parse(*b"ab"), Ok(b'b'));
+    /// assert!(letter_up.parse(*b"ac").is_err());
+    /// ```
+    fn then_with<U, P, F: Fn(O) -> P>(self, other: F) -> ThenWith<I, O, U, Self, P, F>
+    where
+        Self: Sized,
+        P: Parser<I, U, Error = Self::Error>,
+    {
+        ThenWith(self, other, PhantomData)
+    }
+
     /// Parse one thing and then another thing, attempting to chain the two outputs into a [`Vec`].
     ///
     /// The output type of this parser is `Vec<T>`, composed of the elements of the outputs of both parsers.
@@ -814,7 +840,7 @@ pub trait Parser<I: Clone, O> {
     ///     .padded()
     ///     .repeated()
     ///     .map(SExpr::List)
-    ///     .delimited_by('(', ')')
+    ///     .delimited_by(just('('), just(')'))
     ///     .or(ident.map(SExpr::Ident))
     ///     .or(num.map(SExpr::Num)));
     ///
@@ -835,12 +861,18 @@ pub trait Parser<I: Clone, O> {
     ///     ),
     /// );
     /// ```
-    fn delimited_by(self, start: I, end: I) -> DelimitedBy<Self, I>
+    fn delimited_by<U, V, L, R>(self, start: L, end: R) -> DelimitedBy<Self, L, R, U, V>
     where
         Self: Sized,
-        I: PartialEq,
+        L: Parser<I, U, Error = Self::Error>,
+        R: Parser<I, V, Error = Self::Error>,
     {
-        DelimitedBy(self, start, end)
+        DelimitedBy {
+            item: self,
+            start,
+            end,
+            phantom: PhantomData,
+        }
     }
 
     /// Parse one thing or, on failure, another thing.
@@ -908,7 +940,7 @@ pub trait Parser<I: Clone, O> {
     ///
     /// let expr = recursive::<_, _, _, _, Simple<char>>(|expr| expr
     ///     .separated_by(just(','))
-    ///     .delimited_by('[', ']')
+    ///     .delimited_by(just('['), just(']'))
     ///     .map(Expr::List)
     ///     // If parsing a list expression fails, recover at the next delimiter, generating an error AST node
     ///     .recover_with(nested_delimiters('[', ']', [], |_| Expr::Error))
@@ -1058,7 +1090,7 @@ pub trait Parser<I: Clone, O> {
     ///
     /// - Dynamically building up parsers at runtime
     ///
-    /// - Improving compilation times (Rust can struggle to compile code containing very long times)
+    /// - Improving compilation times (Rust can struggle to compile code containing very long types)
     ///
     /// - Passing a parser over an FFI boundary
     ///
@@ -1346,7 +1378,7 @@ impl<'a, I: Clone, O, E: Error<I>> Parser<I, O> for BoxedParser<'a, I, O, E> {
 ///
 ///     literal.or(ast
 ///         .repeated()
-///         .delimited_by(Token::LParen, Token::RParen)
+///         .delimited_by(just(Token::LParen), just(Token::RParen))
 ///         .map(Ast::List))
 /// });
 ///
